﻿using bachelorAPI_main.Models;
using Microsoft.AspNetCore.Mvc;
using bachelorAPI_main.Handlers;
using bachelorAPI_main.DTOs;

namespace bachelorAPI_main.Controllers
{
    [ApiController]
    [Route("[controller]")]
    [Produces("application/json")]
    public class FeedbackController : ControllerBase
    {
        private CommentHandler _commentHandler;
        private FeedbackHandler _feedbackHandler;
        private ModuleHandler _moduleHandler;
        private CustomerHandler _customerHandler;
        private FeedbackCategoryHandler _feedbackCategoryHandler;

        public FeedbackController(IServiceProvider services)
        {
            _commentHandler = services.GetRequiredService<CommentHandler>();
            _feedbackHandler = services.GetRequiredService<FeedbackHandler>();
            _moduleHandler = services.GetRequiredService<ModuleHandler>();
            _customerHandler = services.GetRequiredService<CustomerHandler>();
            _feedbackCategoryHandler = services.GetRequiredService<FeedbackCategoryHandler>();
        }


        /// <summary>
        /// Retrieves a list of all feedbacks.
        /// </summary>
        /// <remarks>
        /// Retrieves a list of all feedbacks.
        /// </remarks>
        /// <response code="200">Returns all feedbacks</response>
        /// <response code="500">Internal server error</response>

        [HttpGet]
        public async Task<IActionResult> DoGetFeedbacks()
        {
            DTOFeedbackGetResponse response = new();


            try
            {

                response.feedbacks = await _feedbackHandler.GetAllFeedbacks();

                response.Message = "Successfully retrieved all feedbacks.";
                return Ok(response);

            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                response.Message = "Internal server error.";
                return StatusCode(StatusCodes.Status500InternalServerError, response);
            }
        }



        /// <summary>
        /// Adds a feedback to the database. 
        /// </summary>
        /// <remarks>
        ///  CustomerID, ModuleID and FeedbackCategoryID must exist in the database. Correct feedback number must be provided.
        ///  Feedbacknumber ranges from 0 - 4. 
        ///  Feedbacknumber = 0 is Worst and Feedbacknumber = 4 is Best.
        /// </remarks>
        /// <param name="feedback"></param>
        /// <returns>A newly created feedback</returns>
        /// <response code="200">Feedback added successfully.</response>
        /// <response code="400">Bad request. CustomerID, ModuleID and CategoryID must be valid. Feedbacknumber must be between 0 and 4.</response>
        /// <response code="500">Internal server error. CustomerID, ModuleID and CategoryID must exist in the database.</response>

        [HttpPost]
        public async Task<IActionResult> DoPostFeedback([FromBody] DTOFeedbackNewRequest feedback)
        {
            DTOFeedbackResponse response = new();


            try
            {
                if (!ValidInput(feedback))
                {
                    response.Message = $"Problem adding Feedback with CustomerID: {feedback.CustomerSeq}, ModuleID: {feedback.ModuleSeq}, FeedbackCategoryID: {feedback.CategorySeq} and Feedback: {feedback.FeedbackNumber}. " +
                       "Feedbacknumber should be between 0 and 4.";
                    return BadRequest(response.Message);
                }

                var feedbackItem = new Feedback();
                var commentItem = new Comment
                {
                    CommentText = feedback.CommentText
                };

                if (commentItem.CommentText != null)
                {

                    await _commentHandler.AddComment(commentItem);
                    feedbackItem.CommentSeq = commentItem.CommentSeq;

                }
                else
                {
                    feedbackItem.CommentSeq = null;
                }
                feedbackItem.CustomerSeq = feedback.CustomerSeq;
                feedbackItem.FeedbackNumber = (double)feedback.FeedbackNumber;
                feedbackItem.CategorySeq = feedback.CategorySeq;
                feedbackItem.ModuleSeq = feedback.ModuleSeq;
                feedbackItem.FeedbackSince = DateTime.Now;

                var result = await _feedbackHandler.AddFeedback(feedbackItem);

                response.FeedbackSeq = result.FeedbackSeq;
                response.Message = "Feedback added successfully.";
                return Ok(response.Message);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                response.Message = $"Internal server error. CustomerID: {feedback.CustomerSeq}, " +
                    $"ModuleID: {feedback.ModuleSeq} and FeedbackCategoryID: {feedback.CategorySeq} must exist in the database.";

                return StatusCode(StatusCodes.Status500InternalServerError, response.Message);
            }

        }


        /// <summary>
        /// Checks if CustomerID, ModuleID, CatagoryID and FeedbackNumber has valid input
        /// </summary>
        /// <param name="feedback"></param>
        /// <returns>True if input is valid false otherwise</returns>
        private bool ValidInput(DTOFeedbackNewRequest feedback)
        {
            return feedback.CustomerSeq > 0 &&
                feedback.ModuleSeq > 0 &&
                feedback.CategorySeq > 0
                && Enum.IsDefined(typeof(FeedbackNumberEnum), feedback.FeedbackNumber);

        }


    }
}

